type JsonObject = {
    [Key in string]: JsonValue;
} & {
    [Key in string]?: JsonValue | undefined;
};
type JsonArray = JsonValue[] | readonly JsonValue[];
type JsonPrimitive = string | number | boolean | null;
type JsonValue = JsonPrimitive | JsonObject | JsonArray;
interface BaseOperation {
    path: string;
}
interface AddOperation<T = any> extends BaseOperation {
    op: "add";
    value: T;
}
interface RemoveOperation extends BaseOperation {
    op: "remove";
}
interface ReplaceOperation<T = any> extends BaseOperation {
    op: "replace";
    value: T;
}
interface MoveOperation extends BaseOperation {
    op: "move";
    from: string;
}
interface CopyOperation extends BaseOperation {
    op: "copy";
    from: string;
}
interface TestOperation<T = any> extends BaseOperation {
    op: "test";
    value: T;
}
type Operation = AddOperation | RemoveOperation | ReplaceOperation | MoveOperation | CopyOperation | TestOperation;
type Patch = Operation[];
type Direction = 'left' | 'right';
interface Config {
    comparator?: (obj: JsonValue, direction: Direction) => string;
}
declare function generateJsonPatch(before: JsonValue, after: JsonValue, config?: Config): Patch;

export { AddOperation, BaseOperation, CopyOperation, MoveOperation, Operation, Patch, RemoveOperation, ReplaceOperation, TestOperation, generateJsonPatch };
