// src/index.ts
function generateJsonPatch(before, after, config = {}) {
  const { comparator } = config;
  const patch = [];
  function isScalar(value) {
    return value === null || typeof value === "undefined" || typeof value === "string" || typeof value === "number" || typeof value === "boolean";
  }
  function getComparatorHash(obj, direction) {
    if (comparator && typeof obj === "object" && obj !== null) {
      return comparator(obj, direction);
    }
    return "unknown";
  }
  function compareArrayByIndex(arr1, arr2, newPath) {
    let currentIndex = 0;
    const maxLength = Math.max(arr1.length, arr2.length);
    for (let i = 0; i < maxLength; i++) {
      const newPathIndex = `${newPath}/${currentIndex++}`;
      if (i < arr1.length && i < arr2.length) {
        compareObjects(newPathIndex, arr1[i], arr2[i]);
      } else if (i >= arr1.length && i < arr2.length) {
        patch.push({ op: "add", path: newPathIndex, value: arr2[i] });
      } else if (i < arr1.length && i >= arr2.length) {
        patch.push({ op: "remove", path: newPathIndex });
        currentIndex--;
      }
    }
  }
  function compareArrayByHash(arr1, arr2, newPath) {
    if (!getComparatorHash) {
      throw Error("No hash function provided");
    }
    const arr1Hashes = arr1.map((value) => getComparatorHash(value, "left"));
    const arr2Hashes = arr2.map((value) => getComparatorHash(value, "right"));
    let currentIndex = 0;
    const notMatchedIndex = [];
    const shouldMove = [];
    for (let i = 0; i < arr1Hashes.length; i++) {
      const newPathIndex = `${newPath}/${currentIndex++}`;
      const arr2HashIndex = arr2Hashes.indexOf(arr1Hashes[i]);
      if (arr2HashIndex >= 0) {
        compareObjects(newPathIndex, arr1[i], arr2[arr2HashIndex]);
        if (i !== arr2HashIndex) {
          shouldMove.push(arr1Hashes[i]);
        }
      } else {
        patch.push({ op: "add", path: newPathIndex, value: arr2[i] });
      }
    }
    for (const i of notMatchedIndex) {
      patch.push({ op: "remove", path: `${newPath}/${i}` });
    }
  }
  function compareArrays(arr1, arr2, newPath) {
    if (comparator) {
      compareArrayByHash(arr1, arr2, newPath);
    } else {
      compareArrayByIndex(arr1, arr2, newPath);
    }
  }
  function compareObjects(path, o1, o2) {
    const isArrayAtTop = path === "" && (Array.isArray(o1) || Array.isArray(o2));
    if (isScalar(o1) && isScalar(o2)) {
      if (o1 !== o2) {
        patch.push({ op: "replace", path, value: o2 });
      }
      return;
    }
    if (isArrayAtTop) {
      return compareArrays(o1, o2, "");
    }
    for (const key in o2) {
      let newPath = isArrayAtTop && path === "" ? `/${key}` : `${path}/${key}`;
      const obj1Value = o1[key];
      const obj2Value = o2[key];
      if (Array.isArray(obj1Value) && Array.isArray(obj2Value)) {
        compareArrays(obj1Value, obj2Value, newPath);
      } else if (typeof obj2Value === "object" && obj2Value !== null) {
        if (typeof obj1Value === "object" && obj1Value !== null) {
          compareObjects(newPath, obj1Value, obj2Value);
        } else {
          patch.push({ op: "replace", path: newPath, value: obj2Value });
        }
      } else if (!o1.hasOwnProperty(key)) {
        patch.push({ op: "add", path: newPath, value: obj2Value });
      } else if (obj1Value !== obj2Value) {
        patch.push({ op: "replace", path: newPath, value: obj2Value });
      }
    }
    for (const key in o1) {
      if (!o1.hasOwnProperty(key))
        continue;
      if (!o2.hasOwnProperty(key)) {
        let newPath = isArrayAtTop && path === "" ? `/${key}` : `${path}/${key}`;
        patch.push({ op: "remove", path: newPath });
      }
    }
  }
  compareObjects("", before, after);
  return [...patch];
}
export {
  generateJsonPatch
};
